; adc_dac_int_jan11.a51  : splices ADC_TEST with DAC_TEST, to do  in-out on interrupt 
; redone using Wizard, pins reassigned Jan 11

; rev path 3/16
	$NOSYMBOLS		; keeps listing short
	$INCLUDE (C:\SiLabs\MCU\INC\c8051f410.inc) 
	$INCLUDE (C:\SiLabs\MCU\INC\VECTORS320.INC)	; Tom's vectors definition file
                                                  ; this you must install by hand

	STACKBOT EQU 080h					; put stack at start of scratch indirectly-addressable block (80h and up)
	SOFTFLAG EQU 0						; software flag at bit 0: used by ISR to say 'time to sample'

	CNVRSN_DONE EQU ADC0CN.5	; flag bit indicates ADC sample is ready
	CNVRT_START EQU ADC0CN.4	; ADC start bit: needs low-going pulse

	DISPLAY_HI EQU P1					; display high 8 bits of 12

	SAMPLE_HI EQU R7

	; INT0 at P0.5, to signal time to take a sample
	; ADC0 at PO.0
	; DAC1 at PO.1


	ORG 0h
	LJMP STARTUP        

	ORG 080h
				
STARTUP: mov SP, #STACKBOT-1

	acall USUAL_SETUP

   acall Init_Device
   acall INT_EDGE      ; make INT1 edge-sensitive
   acall PORT_SETUP     ; set bits high, to free them for ADC and DAC

CHECK_FOR:	jnb SOFTFLAG, $	; hang here till interrupt says 'time to transfer'
	clr SOFTFLAG
	clr EA						; disable interrupts, to make sure it get played out to DAC

        acall GET_SAMPLE

	mov DISPLAY_HI, SAMPLE_HI	; this to let us watch as it proceeds

	mov IDA1H, SAMPLE_HI
       setb EA						; re-enable interrupts
	sjmp CHECK_FOR



;--- SUBROUTINES ----

; ISR1: This is response to INT ONE: set softflag
	ORG INT1VECTOR				; this is defined in VECTORS3210.INC, included above.
						;  It is address 13h, the address to which micro hops
						;   in response to interrupt ONE

ISR0: setb SOFTFLAG	; pseudo-polling: a flag that MAIN will check
	RETI	

GET_SAMPLE: 	CLR CNVRT_START		;  low on AD0BUSY (to permit rising edge)
          	SETB CNVRT_START		;  rising edge on ADOBUSY starts conversion
            	JNB CNVRSN_DONE, $; hang here till conversion-done flag
							CLR CNVRSN_DONE		; not required, but tidy to clear this flag
            	MOV SAMPLE_HI, ADC0H		; high byte
            	RET


; ------INITS ---------
USUAL_SETUP: anl   PCA0MD, #NOT(040h)        ; Disable the WDT.
                                      ; Clear Watchdog Enable bit
				      ; Configure the Oscillator

         mov OSCICN, #087h	      ; max speed sysclk: 24.5 Mhz 
         ret

INT_EDGE: setb IT0		; make INT0 Edge-sensitive
    ret

PORT_SETUP: setb P0.0				; make sure latch is high (this for DAC)
			setb P0.1		; ...and this for ADC
    ret

;------------------------------------
;-  Generated Initialization File  --
;------------------------------------


public  Init_Device

INIT SEGMENT CODE
    rseg INIT

; Peripheral specific initialization functions,
; Called from the Init_Device label
ADC_Init:
    mov  ADC0MX,    #000h
    mov  ADC0CN,    #084h
    ret

DAC_Init:
    mov  IDA1CN,    #0F2h
    ret

Voltage_Reference_Init:
    mov  REF0CN,    #013h
    ret

Port_IO_Init:
    ; P0.0  -  Skipped,     Open-Drain, Analog
    ; P0.1  -  Skipped,     Open-Drain, Analog
    ; P0.2  -  Unassigned,  Open-Drain, Digital
    ; P0.3  -  Unassigned,  Open-Drain, Digital
    ; P0.4  -  Unassigned,  Open-Drain, Digital
    ; P0.5  -  Unassigned,  Open-Drain, Digital
    ; P0.6  -  Skipped,     Open-Drain, Digital
    ; P0.7  -  Unassigned,  Open-Drain, Digital

    ; P1.0  -  Unassigned,  Open-Drain, Digital
    ; P1.1  -  Unassigned,  Open-Drain, Digital
    ; P1.2  -  Skipped,     Open-Drain, Digital
    ; P1.3  -  Unassigned,  Open-Drain, Digital
    ; P1.4  -  Unassigned,  Open-Drain, Digital
    ; P1.5  -  Unassigned,  Open-Drain, Digital
    ; P1.6  -  Unassigned,  Open-Drain, Digital
    ; P1.7  -  Unassigned,  Open-Drain, Digital

    ; P2.0  -  Unassigned,  Open-Drain, Digital
    ; P2.1  -  Unassigned,  Open-Drain, Digital
    ; P2.2  -  Unassigned,  Open-Drain, Digital
    ; P2.3  -  Unassigned,  Open-Drain, Digital
    ; P2.4  -  Unassigned,  Open-Drain, Digital
    ; P2.5  -  Unassigned,  Open-Drain, Digital
    ; P2.6  -  Unassigned,  Open-Drain, Digital
    ; P2.7  -  Unassigned,  Open-Drain, Digital

    mov  P0MDIN,    #0FCh
    mov  P0SKIP,    #043h
    mov  P1SKIP,    #004h
    mov  XBR1,      #040h
    ret


Interrupts_Init:
    mov  IT01CF,    #005h
    mov  IE,        #081h  ; enable INT0* and global interrupts
    ret

; Initialization function for device,
; Call Init_Device from your main program
Init_Device:
    lcall ADC_Init
    lcall DAC_Init
    lcall Voltage_Reference_Init
    lcall Port_IO_Init
    lcall Interrupts_Init
    ret

end


END

